@@ -41,6 +41,7 @@
 
 #define GET_ROUNDED_BUTTONS(style) (INDUSTRIAL_STYLE (style)->rounded_buttons)
 #define GET_REAL_OPACITY(style, orig_opacity) (CLAMP ((orig_opacity) * INDUSTRIAL_STYLE (style)->contrast, 0.0, 1.0))
+#define GET_HINT(style) (INDUSTRIAL_STYLE (style)->hint)
 
 #define IF_ROUNDED(style, rounded, otherwise) (GET_ROUNDED_BUTTONS (style) ? rounded : otherwise)
 
@@ -164,7 +165,7 @@ draw_rounded_gradient (cairo_t    *cr,
 	
 	/* cairo_rectangle (cr, outer_radius, 0, width - 2 * outer_radius,
 			 gradient_width); */
-	cairo_matrix_init_rotate (&matrix, M_PI);
+	cairo_matrix_init_rotate (&matrix, G_PI);
 	cairo_matrix_translate (&matrix, 0, -gradient_width);
 	cairo_pattern_set_matrix (pattern, &matrix);
 	cairo_fill (cr);
@@ -183,7 +184,7 @@ draw_rounded_gradient (cairo_t    *cr,
 	
 	/* cairo_rectangle (cr, width - outer_radius, outer_radius, outer_radius,
 			 height - 2 * outer_radius); */
-	cairo_matrix_init_rotate (&matrix, M_PI_2);
+	cairo_matrix_init_rotate (&matrix, G_PI_2);
 	cairo_matrix_translate (&matrix, -width + gradient_width, -gradient_width);
 	cairo_pattern_set_matrix (pattern, &matrix);
 	cairo_fill (cr);
@@ -220,7 +221,7 @@ draw_rounded_gradient (cairo_t    *cr,
 
 	/* cairo_rectangle (cr, 0, outer_radius, gradient_width,
 			 height - 2 * outer_radius); */
-	cairo_matrix_init_rotate (&matrix, -M_PI_2);
+	cairo_matrix_init_rotate (&matrix, -G_PI_2);
 	cairo_matrix_translate (&matrix, -gradient_width, 0);
 	cairo_pattern_set_matrix (pattern, &matrix);
 	cairo_fill (cr);
@@ -584,10 +585,62 @@ real_draw_box (GtkStyle      *style,
 	 * important, for unkown stuff we fall back to flat_box. */
 	cairo_save (cr);
 
-	if (CHECK_DETAIL (detail, "button") ||
-	    CHECK_DETAIL (detail, "togglebutton") ||
-	    CHECK_DETAIL (detail, "optionmenu")) {
-		/* Draw the button border, except if this is a "default" button. */
+	if (CHECK_DETAIL (detail, "button") && ge_check_hint (GE_HINT_TREEVIEW_HEADER, GET_HINT (style), widget)) {
+		CairoColor bg;
+		CairoColor fg;
+		
+		ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
+		ge_gdk_color_to_cairo (&style->fg[state_type], &fg);
+		
+		cairo_rectangle (cr, x, y, width, height);
+		ge_cairo_set_color (cr, &bg);
+		cairo_fill (cr);
+		
+		if (shadow_type == GTK_SHADOW_OUT) {
+			CairoColor inner, outer;
+			cairo_pattern_t *pattern;
+			
+			ge_gdk_color_to_cairo (&style->fg[state_type], &inner);
+			ge_gdk_color_to_cairo (&style->fg[state_type], &outer);
+			inner.a = 0;
+			outer.a = 0.1; /* XXX: Same as for buttons! */
+			
+			pattern = cairo_pattern_create_linear (x, height - 5, x, height - 1);
+			ge_cairo_pattern_add_color_stop_color (pattern, 0.0, &inner);
+			ge_cairo_pattern_add_color_stop_color (pattern, 1.0, &outer);
+			cairo_set_source (cr, pattern);
+			
+			cairo_rectangle (cr, x, height - 5, width, 4.0);
+			cairo_fill (cr);
+			cairo_pattern_destroy (pattern);
+
+
+			cairo_move_to (cr, x + 0.5, y + 0.5);
+			cairo_line_to (cr, x + width - 0.5, y + 0.5);
+			fg.a = BUTTON_BORDER_OPACITY * 0.05;
+			ge_cairo_set_color (cr, &fg);
+			cairo_stroke (cr);
+		}
+
+		cairo_move_to (cr, x + 0.5, height - 0.5);
+		cairo_line_to (cr, x + width - 0.5, height - 0.5);
+		fg.a = BUTTON_BORDER_OPACITY;
+		ge_cairo_set_color (cr, &fg);
+		cairo_stroke (cr);
+		
+		/* XXX: do not draw the line for the last column, if the headers are not resizeable. */
+		if (ge_widget_is_ltr (widget)) {
+			cairo_move_to (cr, x + width - 1.5, y + 5.5); /* 1.5 or 0.5? */
+			cairo_line_to (cr, x + width - 1.5, y + height - 5.5);
+		} else {
+			cairo_move_to (cr, x + 1.5, y + 5.5);
+			cairo_line_to (cr, x + 1.5, y + height - 5.5);
+		}
+		cairo_stroke (cr);
+		
+	} else if (CHECK_DETAIL (detail, "button") ||
+	           CHECK_DETAIL (detail, "togglebutton") ||
+	           CHECK_DETAIL (detail, "optionmenu")) {
 		CairoColor bg;
 		CairoColor fg;
 		CairoCorners corners = CR_CORNER_ALL;
@@ -604,7 +657,7 @@ real_draw_box (GtkStyle      *style,
 			fg.a = GET_REAL_OPACITY (style, BUTTON_BORDER_OPACITY);
 
 
-		if (ge_is_in_combo_box (widget)) {
+		if (ge_check_hint (GE_HINT_COMBOBOX_ENTRY, GET_HINT (style), widget)) {
 			if (ge_widget_is_ltr (widget))
 				corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
 			else
@@ -669,8 +722,6 @@ real_draw_box (GtkStyle      *style,
 			cairo_close_path (cr);
 			cairo_clip (cr);
 
-			/* XXX: This results in a few pixel that stay blank.
-			 * Need to rethink this one. */
 			draw_rounded_gradient (cr, x + 1, y + 1,
 					       width - 2, height - 2, 4, 0,
 					       IF_ROUNDED (style, 2.5, 0), &inner, &outer);
@@ -753,6 +804,8 @@ real_draw_box (GtkStyle      *style,
 	} else if (CHECK_DETAIL (detail, "trough")
 		   || CHECK_DETAIL (detail, "trough-lower")
 		   || CHECK_DETAIL (detail, "trough-upper")
+		   || CHECK_DETAIL (detail, "trough-fill-level")
+		   || CHECK_DETAIL (detail, "trough-fill-level-full")
 		   || CHECK_DETAIL (detail, "menu")
 		   || CHECK_DETAIL (detail, "dockitem_bin")
 		   || CHECK_DETAIL (detail, "handlebox_bin")
@@ -765,7 +818,9 @@ real_draw_box (GtkStyle      *style,
 		if (fill) {
 			if (CHECK_DETAIL (detail, "trough") ||
 			    CHECK_DETAIL (detail, "trough-lower") ||
-			    CHECK_DETAIL (detail, "trough-upper")) {
+			    CHECK_DETAIL (detail, "trough-upper") ||
+			    CHECK_DETAIL (detail, "trough-fill-level") ||
+			    CHECK_DETAIL (detail, "trough-fill-level-full")) {
 				CairoColor bg_color;
 
 				/* Troughs cannot be transparent!
@@ -782,7 +837,8 @@ real_draw_box (GtkStyle      *style,
 				}
 				
 				/* XXX: This might need improvement, at least now it is "implemented" */
-				if (CHECK_DETAIL (detail, "trough-lower")) {
+				if (CHECK_DETAIL (detail, "trough-lower") ||
+				    CHECK_DETAIL (detail, "trough-fill-level") || CHECK_DETAIL (detail, "trough-fill-level-full")) {
 					ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &bg);
 				} else {
 					ge_gdk_color_to_cairo (&style->fg[state_type], &bg);
@@ -867,8 +923,8 @@ real_draw_box (GtkStyle      *style,
 		CairoColor bevel, bg;
 		CairoCorners corners = CR_CORNER_ALL;
 
-		if ((GE_IS_SPIN_BUTTON (widget) && CHECK_DETAIL (detail, "entry")) ||
-		    ge_is_in_combo_box (widget)) {
+		if ((CHECK_DETAIL (detail, "entry") && ge_check_hint (GE_HINT_SPINBUTTON, GET_HINT (style), widget)) ||
+		    ge_check_hint (GE_HINT_COMBOBOX_ENTRY, GET_HINT (style), widget)) {
 			/* effectively cut one side off. */
 			width += 2;
 			if (!ge_widget_is_ltr (widget))
@@ -953,7 +1009,8 @@ draw_focus (GtkStyle      *style,
 		SANITIZE_SIZE
 		CHECK_ARGS
 
-		if (ge_is_in_combo_box (widget) || GE_IS_SPIN_BUTTON (widget)) {
+		if (ge_check_hint (GE_HINT_COMBOBOX_ENTRY, GET_HINT (style), widget) ||
+		    ge_check_hint (GE_HINT_SPINBUTTON, GET_HINT (style), widget)) {
 			if (ge_widget_is_ltr (widget))
 				corners = CR_CORNER_TOPLEFT | CR_CORNER_BOTTOMLEFT;
 			else
@@ -1386,7 +1443,7 @@ draw_option (GtkStyle * style,
 
 	xc = x + width / 2.0;
 	yc = y + height / 2.0;
-	cairo_arc (cr, xc, yc, radius - 0.5, 0, 2 * M_PI);
+	cairo_arc (cr, xc, yc, radius - 0.5, 0, 2 * G_PI);
 
 	ge_cairo_set_color (cr, &bg);
 	cairo_fill (cr);
@@ -1399,8 +1456,8 @@ draw_option (GtkStyle * style,
 	
 	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
 	
-	cairo_arc (cr, xc, yc, radius, 0, 2 * M_PI);
-	cairo_arc (cr, xc + 0.2, yc + 0.2, radius - 1.2, 0, 2 * M_PI);
+	cairo_arc (cr, xc, yc, radius, 0, 2 * G_PI);
+	cairo_arc (cr, xc + 0.2, yc + 0.2, radius - 1.2, 0, 2 * G_PI);
 
 	cairo_fill (cr);
 	
@@ -1411,7 +1468,7 @@ draw_option (GtkStyle * style,
 		fg.a = GET_REAL_OPACITY (style, 1.0);
 
 		ge_cairo_set_color (cr, &fg);
-		cairo_arc (cr, xc, yc, radius - 3, 0, 2 * M_PI);
+		cairo_arc (cr, xc, yc, radius - 3, 0, 2 * G_PI);
 		cairo_fill (cr);
 
 		pattern =
@@ -1426,8 +1483,8 @@ draw_option (GtkStyle * style,
 		cairo_pattern_destroy (pattern);
 
 		cairo_move_to (cr, xc, yc);
-		cairo_arc (cr, xc, yc, radius - 4, M_PI - M_PI_4,
-			   3 * M_PI_2 + M_PI_4);
+		cairo_arc (cr, xc, yc, radius - 4, G_PI - G_PI_4,
+			   3 * G_PI_2 + G_PI_4);
 		cairo_close_path (cr);
 		cairo_fill (cr);
 	} else if (shadow_type == GTK_SHADOW_ETCHED_IN) {
